library AX25Mail;

{*---------------------------------------------------------------------*}
{* AX25Mail                                                            *}
{* This DLL allows the functions to actually deal with the mail        *}
{* to be encapsulated so replacing it might allow the package to be    *}
{* used with another BBS program                                       *}
{*---------------------------------------------------------------------*}
{*        Copyright (c) LU7DID 2000, Free for Radioamateur uses        *}
{*         Ing. Pedro E. Colla (LU7DID)  colla@pec.pccp.com.ar         *}
{*---------------------------------------------------------------------*}

uses
  SysUtils,WinTypes,ReqLib32,HexStr,INIFiles;



{*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*}
{*                      Define Constants                                *}
{*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*}

  Const
                                            {*------------------------------*}
                                            {*  Program Related Constants   *}
                                            {*------------------------------*}
    CR          =       Chr($0D);           {*Commonly Used ASCII constants *}
    LF          =       Chr($0A);           {*                              *}
    ESC         =       Chr($1B);           {*                              *}
    TAB         =       Chr($09);           {*                              *}
    CRLF        =       CR+LF;              {*                              *}
    NUL         =       Chr($00);           {*                              *}
                                            {*------------------------------*}
                                            {*  Program Related Constants   *}
                                            {*------------------------------*}
    PROGRAMNAME =       'AX25Mail';         {*Program Name                  *}
    VERSION     =       '0.1A beta';        {*Level                         *}
    AUTHOR      =       'Pedro E. Colla (LU7DID) 1999,2000';
    PROGRAMBUILD=       '00';               {*Build                         *}
    COPYRIGHT   =       'Version '+VERSION+' (c) '+AUTHOR;
                                            {*                              *}
                                            {*------------------------------*}
{*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*}
{*                      Memory Structures                               *}
{*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*}

{*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*}
{*                      Global Memory Allocations                       *}
{*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*}

  Type

  PtrMail          =   ^MailRec;
                                            {*------------------------------*}
  MailRec  = Packed Record                  {*SMTP Connection Entry         *}
    szMailType     :   String;              {*                              *}
    szMailStatus   :   String;              {*                              *}
    iMailSeq       :   Integer;             {*                              *}
    iMailNer       :   Integer;             {*                              *}
    szMailTo       :   String;              {*                              *}
    szMailBBS      :   String;              {*                              *}
    szMailFrom     :   String;              {*                              *}
    szBID          :   String;              {*                              *}
    szSubject      :   String;              {*                              *}
    szFile         :   String;              {*                              *}
    lpNext         :   PtrMail;             {*                              *}
  End;                                      {*                              *}
                                            {*                              *}
                                            {*------------------------------*}
  PtrNews          =   ^NewsList;
                                            {*------------------------------*}
  NewsList = Packed Record                  {*SMTP Connection Entry         *}
    szNewsId       :   String;              {*                              *}
    iLast          :   Integer;             {*                              *}
    iFirst         :   Integer;             {*                              *}
    iNum           :   Integer;             {*                              *}
    iLastSeq       :   Integer;             {*                              *}
    szPost         :   String;              {*                              *}
    lpMailBox      :   PtrMail;             {*                              *}
    lpNext         :   PtrNews;             {*                              *}
  End;                                      {*                              *}
                                            {*                              *}
                                            {*------------------------------*}

  PtrMBox          =   ^MailBox;
                                            {*------------------------------*}
  MailBox  = Packed Record                  {*SMTP Connection Entry         *}
    szMailBox      :   String;              {*                              *}
    szPass         :   String;              {*                              *}
    iMaxMail       :   Integer;             {*                              *}
    iMaxNews       :   Integer;             {*                              *}
    lpMailBox      :   PtrMail;             {*                              *}
    lpNews         :   PtrNews;             {*                              *}
    szGroup        :   String;              {*                              *}
    szArticle      :   String;              {*                              *}
    lpNext         :   PtrMBox;             {*                              *}
  End;                                      {*                              *}
                                            {*                              *}
                                            {*------------------------------*}
  Var
                                            {*------------------------------*}
    bEnabled    :       Boolean;            {*L4 Circuit ID Counter         *}
                                            {*------------------------------*}
    DEBUGMODE   :       String;
    TRACELEVEL  :       Integer;

    lpMbx       :       PtrMbox;
    HAMMAILINI  :       TINIFile;
    szMailDB    :       String;
    szMailDir   :       String;
    szPassVfy   :       String;
    szLogFile   :       String;
    szHomeBBS   :       String;


{*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*}
{*                      GENERAL PURPOSE ROUTINES                        *}
{*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*}

{*--->>  PutLog         <<---------------------------------------------*}
{* Record a string into the Log for debugging purposes   -DEBUG & TEST-*}
{*---------------------------------------------------------------------*}
Procedure PutLog (LogLevel : Byte; LogStr : String);
  Var
    LogFile             :       Text;
    SysDateTime         :       TDatetime;

begin { PutLog }

  If DEBUGMODE <> 'YES' then begin
     Exit;
  end; {endif}

  If LogLevel > TRACELEVEL then begin
     Exit;
  end; {endif}

  Assign(LogFile,szLogFile);

{$I-}
  Append(LogFile);
  If IOResult <> 0 then begin
     Rewrite(LogFile);
  end; {endif}
{$I+}
  SysDateTime := Now;
  WriteLn(LogFile,datetostr(SysDateTime)+' '+timetostr(SysDateTime)+' '+LogStr);
  Close(LogFile);

end; { PutLog }

{*-------------------------------------------------------------------------*}
{* DumpHex                                                                 *}
{* Procedure to dump an hexa list of a buffer                              *}
{*-------------------------------------------------------------------------*}
Procedure DumpHex (LogLevel : Byte ; DumpStr : String);
  Var
    Index       :       Word;
    AuxStr      :       String;
    HexSt       :       String;
    LogFile     :       Text;

begin { DumpHex }

    If DEBUGMODE <> 'YES' then begin
       Exit;
    end; {endif}

    If LogLevel > TRACELEVEL then begin
       Exit;
    end; {endif}

    If Length(DumpStr) < 1 then begin
       Exit;
    end; {endif}

    Assign(LogFile,szLogFile);

{$I-}
    Append(LogFile);
    If IOResult <> 0 then begin
       Rewrite(LogFile);
    end; {endif}
{$I+}
    Index  := 1;
    AuxStr := '';
    HexSt  := '';
    While Index <= Length(DumpStr) do begin
       If (Ord(DumpStr[Index]) < $20) or
          (Ord(DumpStr[Index]) > $7A) then begin
          AuxStr := AuxStr + '.';
          HexSt  := HexSt  + HexByte(Ord(DumpStr[Index])) + ' ';
       end else begin
          AuxStr := AuxStr + DumpStr[Index];
          HexSt  := HexSt  + HexByte(Ord(DumpStr[Index])) + ' ';
       end; {endif}
       If Length(AuxStr) = 16 then begin
          WriteLn(LogFile,' |'+HexSt+'|'+AuxStr);

          AuxStr := '';
          HexSt  := '';
       end; {endif}
       Index := Index + 1;
    end; {endwhile}
    If Length(AuxStr) > 0 then begin

       While Length(AuxStr) < 16 do begin
         AuxStr := AuxStr + ' ';
         HexSt  := HexSt  + '-- ';
       end; {endwhile}

       WriteLn(LogFile,' |'+HexSt+'|'+AuxStr);
    end; {endif}
    Close(LogFile);

end; { DumpHex }
(*--->>> GetNews       <<<----------------------------------------------*)
{* Get an instance of the linked list                                   *}
{*----------------------------------------------------------------------*}
Procedure UpdLastSeq (lpNews : PtrNews; szMailBox : String);

  Var
    NPtr        :       PtrNews;
    MailBoxINI  :       TINIFile;
    iMaxMail    :       Integer;

begin { LL }

 {*----------------------------------------------*}
 {* Read the Configuration Data from HAMMAIL.INI *}
 {*----------------------------------------------*}

   MailBoxINI        := TINIFile.Create('HAMMAIL.INI');
   NPtr := lpNews;
   While NPtr <> Nil do begin
      iMaxMail := strtoint(MailBoxINI.ReadString(szMailBox,NPtr^.szNewsId,'1'));
      NPtr^.iLastSeq := iMaxMail;
      PutLog(5,'{AX25Mail} Updated last read article for mbx('+szMailBox+') Group('+NPtr^.szNewsId+') as ('+IntToStr(NPtr^.iLastSeq)+')');
      NPtr := NPtr.lpNext;
   end; {endwhile}
   MailBoxINI.Free;

end; { LL }
(*--->>> GetLastSeq    <<<----------------------------------------------*)
{* Get an instance of the linked list                                   *}
{*----------------------------------------------------------------------*}
Function GetLastSeq (szMailBox : String; szNewsId : String) : Integer;

  Var
    MailBoxINI  :       TINIFile;
    iMaxMail    :       Integer;

begin { LL }

 {*----------------------------------------------*}
 {* Read the Configuration Data from HAMMAIL.INI *}
 {*----------------------------------------------*}

   MailBoxINI        := TINIFile.Create('HAMMAIL.INI');
   iMaxMail := strtoint(MailBoxINI.ReadString(szMailBox,szNewsId,'0'));
   PutLog(5,'{AX25Mail} Retrieved Last Sequence for MBX('+szMailBox+') NewsGroup('+szNewsId+')');
   MailBoxINI.Free;
   Result   := iMaxMail;

end; { LL }
(*--->>> GetNews       <<<----------------------------------------------*)
{* Get an instance of the linked list                                   *}
{*----------------------------------------------------------------------*}
Function GetNews (lpNews : PtrNews; szNewsId : String) :   PtrNews;

  Var
    NPtr        :       PtrNews;

begin { LL }

    NPtr := lpNews;
    While NPtr <> Nil do begin
       If NPtr.szNewsId = szNewsId then begin
          PutLog(5,'Get Success News Object '+szNewsId);
          Result := NPtr;
          Exit;
       end; {endif}
       NPtr := NPtr.lpNext;
    end; {endwhile}
    PutLog(1,'{AX25Mail} ERROR Get Failure News Object '+szNewsId);
    Result := Nil;

end; { LL }
(*--->>> FreeNews      <<<----------------------------------------------*)
{* Get an instance of the linked list                                   *}
{*----------------------------------------------------------------------*}
Procedure FreeNews (Var lpNews : PtrNews);

  Var
    NPtr        :       PtrNews;
    PPtr        :       PtrNews;

begin { LL }

    If lpNews = Nil then begin
       Exit;
    end; {endif}

    NPtr := lpNews;
    While NPtr <> Nil do begin
       PPtr := NPtr;
       NPtr := NPtr.lpNext;
       Dispose(PPtr);
    end; {endwhile}

end; { LL }
(*--->>> IsrtNews      <<<----------------------------------------------*)
{*Isrt an instance of the linked list                                   *}
{*----------------------------------------------------------------------*}
Function IsrtNews (Var lpNews : PtrNews; szNewsId : String) : PtrNews ;
  Var
    NPtr        :       PtrNews;
    PPtr        :       PtrNews;

begin { LL }

  NPtr := lpNews;
  PPtr := lpNews;

  While PPtr <> Nil do begin
    If (PPtr^.szNewsId = szNewsId) then begin
       Result := PPtr;
       Exit;
    end; {endif}
    PPtr := NPtr^.lpNext;
    If PPtr <> Nil then begin
       NPtr := PPtr;
    end; {endif}
  end; {endwhile}

  New(PPtr);
  PPtr^.szNewsId    := szNewsId;
  PPtr^.iLast       := 0;
  PPtr^.iFirst      := 0;
  PPtr^.iNum        := 0;
  PPtr^.iLastSeq    := 0;
  PPtr^.szPost      := 'y';
  PPtr^.lpMailBox   := Nil;
  PPtr^.lpNext      := Nil;

  If lpNews = Nil then begin
     lpNews := PPtr;
     PutLog(5,'Isrt Success (First) News Object '+szNewsId);
  end else begin
     PutLog(5,'Isrt Success (Next) News Object '+szNewsId);
     NPtr^.lpNext := PPtr;
  end; {endif}

  Result := PPtr;

end; { LL }
(*--->>> DlteNews      <<<----------------------------------------------*)
{*Dlte an instance of the linked list                                   *}
{*----------------------------------------------------------------------*}
Function DlteNews (Var lpNews : PtrNews; szNewsId : String) : PtrNews;
  Var
    NPtr        :       PtrNews;
    PPtr        :       PtrNews;
begin { LL }

   If lpNews = Nil then begin
      PutLog(1,'{AX25Mail} ERROR Dlte Failure (empty list) News Object '+szNewsId);
      Result := Nil;
      Exit;
   end; {endif}

   NPtr := lpNews;
   PPtr := Nil;

   While NPtr <> Nil do begin
      If NPtr^.szNewsId = szNewsId then begin
         If PPtr <> Nil then begin
            PPtr^.lpNext := NPtr^.lpNext;
            PutLog(5,'Dlte Success (last) News Object '+szNewsId);
            Result := NPtr;
            Dispose(NPtr);
            Exit;
         end else begin
            If NPtr^.lpNext <> Nil then begin
               lpNews := NPtr^.lpNext;
               PutLog(5,'Dlte Success (mid) News Object '+szNewsId);
               Result := NPtr;
               Dispose(NPtr);
               Exit;
            end else begin
               PutLog(5,'Dlte Success (last) News Object '+szNewsId);
               Result := NPtr;
               Dispose(NPtr);
               lpNews := Nil;
               Exit;
            end; {endif}
         end; {endif}
         Result := Nil;
         Exit;
      end else begin
         PPtr := NPtr;
         NPtr := NPtr^.lpNext;
      end; {endif}
   end; {endwhile}

   PutLog(1,'{AX25Mail} ERROR Dlte failure (end of list) News Object '+szNewsId);
   Result := Nil;

end; { LL }
{*--->>  ReleaseNewsLL <<----------------------------------------------*}
{* This entry is used to release the memory LL                         *}
{*---------------------------------------------------------------------*}
Procedure ReleaseNewsLL (Var lpHead : PtrNews);
  Var
    NPtr        :       PtrNews;
    PPtr        :       PtrNews;

begin { ReleaseLL }

  NPtr := lpHead;
  While NPtr <> Nil do begin
    PPtr := NPtr;
    NPtr := NPtr^.lpNext;
    Dispose(PPtr);
  end; {endwhile}
  lpHead := Nil;

  PutLog(3,'{AX25Mail} News Linked List Released');

end; { ReleaseLL }
{*-------------------------------------------------------------------------*}
{* ScanDMS                                                                 *}
{* Scan the Index Data Base                                                *}
{*-------------------------------------------------------------------------*}
Procedure ScanDMS (Var LastSeqNum     : Integer;
                       MailType       : String;
                       szFromFilter   : String;
                       szBBSFilter    : String;
                       szNewsGroups   : String;
                       DMSFileName    : String;
                   Var lpHead         : PtrMail;
                       MailDir        : String);

  Label
    NextMssg;

  Var
    DMSFile     :       File;
    BinChar     :       Array [0..260] of Char;
    MssgStr     :       String;
    CmmdStr     :       String;
    StatStr     :       String;
    FromId      :       String;
    ToId        :       String;
    ToBBS       :       String;
    Bid         :       String;
    Subject     :       String;
    MailFileName:       String;
    Aux0        :       Byte;
    Aux1        :       Byte;
    Aux2        :       Byte;
    Aux3        :       Byte;
    Index       :       Integer;
    ReadIn      :       Integer;
    MsgNro      :       Real;
    MsgNroAux   :       Real;
    LastNum     :       Real;
    AuxInt      :       Integer;
    NPtr        :       PtrMail;
    SPtr        :       PtrMail;
    PPtr        :       PtrMail;
    iMailNer    :       Integer;


begin { ScanDMS }

    iMailNer := 1;
    AssignFile(DMSFile,DMSFileName);
    Reset(DMSFile,1);

    PutLog(3,'{AX25Mail} Scan of DMS File '+DMSFileName);
    PutLog(5,'{AX25Mail} MailBox: <'+szFromFilter+'>');
    PutLog(5,'{AX25Mail} Group  : <'+szBBSFilter+'>');
    PutLog(5,'{AX25Mail} Type   : <'+MailType+'>');
    PutLog(5,'{AX25Mail} Start  : <'+IntToStr(LastSeqNum)+'>');

    While not EOF(DMSFile) do begin
       BlockRead(DMSFile,BinChar,128,ReadIn);

       If (MailType = BinChar[0]) or (MailType = '*') then begin

          CmmdStr:= 'S'+BinChar[0];
          StatStr:= BinChar[1];

          Aux0    := Ord(BinChar[2]);
          Aux1    := Ord(BinChar[3]);
          Aux2    := Ord(BinChar[4]);
          Aux3    := Ord(BinChar[5]);

          MsgNro    := Aux0 + (256.0 * Aux1);
          MsgNroAux := (Aux2 + (256.0 * Aux3));
          MsgNroAux := 65536.0 * MsgNroAux;
          MsgNroAux := MsgNro + MsgNroAux;
          MsgNro    := MsgNroAux;

          LastNum := 1.0 * LastSeqNum;
          If MsgNro >= LastNum then begin

             LastSeqNum := Trunc(MsgNro);
             FromId  := '';
             ToId    := '';
             ToBBS   := '';
             Bid     := '';
             Subject := '';

             For Index := 14  to 20  do begin
                 ToId := ToId + BinChar[Index];
             end; {endfor}

             For Index := 21 to 27  do begin
                 ToBBS := ToBBS + BinChar[Index];
             end; {endfor}

             For Index := 28 to 34  do begin
                 FromId := FromId + BinChar[Index];
             end; {endfor}

             For Index := 104 to 116  do begin
                 Bid := Bid + BinChar[Index];
             end; {endfor}

             For Index := 35 to 95 do begin
                 Subject := Subject + BinChar[Index];
             end; {endfor}

             ToId   := GetStrZ(ToId);
             ToBBS  := GetStrZ(ToBBS);
             ToBBS  := Parse(ToBBS);

             FromId := GetStrZ(FromId);
             Bid    := GetStrZ(Bid);
             Subject:= GetStrZ(Subject);

             If Subject = '' then begin
                Subject := 'NO SUBJECT';
             end; {endif}

             AuxInt  := Trunc(MsgNro);
             MssgStr := XPadStr(IntToStr(AuxInt),8,'0');
             MailFileName := MailDir+MssgStr+'.MES';

             If ToBBS = '' then begin
                ToBBS := szHomeBBS;
             end; {endif}

             If ( (BinChar[0]   = 'P' ) and
                 ((szFromFilter = '*' ) or
                  (szFromFilter = ToId)) )

                  or

               ( ((BinChar[0]   = 'B') or (BinChar[0]  = '*'  ))
                 and
                 ((szBBSFilter  = '*') or (szBBSFilter = ToBBS))
                 and
                 ((szNewsGroups = '' ) or (Pos(ToBBS,szNewsGroups) <> 0))
               ) then begin

                NPtr := lpHead;
                SPtr := NPtr;
                While NPtr <> Nil do begin
                   SPtr := NPtr;
                   NPtr := NPtr^.lpNext;
                end; {endwhile}

                New(PPtr);

                PPtr^.szMailType   := CmmdStr;
                PPtr^.szMailStatus := StatStr;
                PPtr^.iMailSeq     := Trunc(MsgNro);
                PPtr^.iMailNer     := iMailNer;
                PPtr^.szMailTo     := ToId;
                PPtr^.szMailBBS    := ToBBS;
                PPtr^.szMailFrom   := FromId;
                PPtr^.szBID        := Bid;
                PPtr^.szSubject    := Subject;
                PPtr^.szFile       := MailFileName;
                PPtr^.lpNext       := Nil;

                inc(iMailNer);

                If lpHead = Nil then begin
                   lpHead := PPtr;
                end else begin
                   SPtr^.lpNext := PPtr;
                end; {endif}

                PutLog(5,'{AX25Mail} ('+IntToStr(Trunc(MsgNro))+') Cmd='+CmmdStr+' '+ToId+' '+ToBBS+' '+FromId+' '+Bid+' '+Subject+' '+MailFileName);

             end; {endif}
          end; {endif}
       end; {endif}

    end; {endwhile}

    PutLog(5,'{AX25Mail} Closing DMS File '+DMSFileName);
    Close(DMSFile);

end; { ScanDMS }
{*-------------------------------------------------------------------------*}
{* SearchDMS                                                               *}
{* Search the DMS environment for a given list of News                     *}
{*-------------------------------------------------------------------------*}
Procedure SearchDMS(Var LastSeqNum     : Integer;
                        MailType       : String;
                        szGroup        : String;
                        szNewsGroups   : String;
                        DMSFileName    : String;
                    Var lpNews         : PtrNews;
                        MailDir        : String);

  Label
    NextMssg;

  Var
    DMSFile     :       File;
    BinChar     :       Array [0..260] of Char;
    MssgStr     :       String;
    CmmdStr     :       String;
    StatStr     :       String;
    FromId      :       String;
    ToId        :       String;
    ToBBS       :       String;
    Bid         :       String;
    Subject     :       String;
    MailFileName:       String;
    Aux0        :       Byte;
    Aux1        :       Byte;
    Aux2        :       Byte;
    Aux3        :       Byte;
    Index       :       Integer;
    ReadIn      :       Integer;
    MsgNro      :       Real;
    MsgNroAux   :       Real;
    LastNum     :       Real;
    AuxInt      :       Integer;

    lpGroup     :       PtrNews;
    szFromFilter:       String;
    szBBSFilter :       String;


begin { ScanDMS }

    PutLog(3,'{AX25Mail} Search of DMS File '+DMSFileName);
    PutLog(5,'{AX25Mail} Type   : <'+MailType+'>');
    PutLog(5,'{AX25Mail} Start  : <'+IntToStr(LastSeqNum)+'>');


    szFromFilter := '*';
    szBBSFilter  := '*';

    AssignFile(DMSFile,DMSFileName);
    Reset(DMSFile,1);

    While not EOF(DMSFile) do begin
       BlockRead(DMSFile,BinChar,128,ReadIn);

       If (MailType = BinChar[0]) or (MailType = '*') then begin

          CmmdStr:= 'S'+BinChar[0];
          StatStr:= BinChar[1];

          Aux0    := Ord(BinChar[2]);
          Aux1    := Ord(BinChar[3]);
          Aux2    := Ord(BinChar[4]);
          Aux3    := Ord(BinChar[5]);

          MsgNro    := Aux0 + (256.0 * Aux1);
          MsgNroAux := (Aux2 + (256.0 * Aux3));
          MsgNroAux := 65536.0 * MsgNroAux;
          MsgNroAux := MsgNro + MsgNroAux;
          MsgNro    := MsgNroAux;

          LastNum := 1.0 * LastSeqNum;
          If MsgNro >= LastNum then begin

             LastSeqNum := Trunc(MsgNro);
             FromId  := '';
             ToId    := '';
             ToBBS   := '';
             Bid     := '';
             Subject := '';

             For Index := 14  to 20  do begin
                 ToId := ToId + BinChar[Index];
             end; {endfor}

             For Index := 21 to 27  do begin
                 ToBBS := ToBBS + BinChar[Index];
             end; {endfor}

             For Index := 28 to 34  do begin
                 FromId := FromId + BinChar[Index];
             end; {endfor}

             For Index := 104 to 116  do begin
                 Bid := Bid + BinChar[Index];
             end; {endfor}

             For Index := 35 to 95 do begin
                 Subject := Subject + BinChar[Index];
             end; {endfor}

             ToId   := GetStrZ(ToId);
             ToBBS  := GetStrZ(ToBBS);
             ToBBS  := Parse(ToBBS);

             FromId := GetStrZ(FromId);
             Bid    := GetStrZ(Bid);
             Subject:= GetStrZ(Subject);

             If Subject = '' then begin
                Subject := 'NO SUBJECT';
             end; {endif}

             AuxInt  := Trunc(MsgNro);
             MssgStr := XPadStr(IntToStr(AuxInt),8,'0');
             MailFileName := MailDir+MssgStr+'.MES';

             If ToBBS = '' then begin
                ToBBS := szHomeBBS;
             end; {endif}

             If ( (BinChar[0]   = 'P' ) and
                 ((szFromFilter = '*' ) or
                  (szFromFilter = ToId)) )

                  or

               ( ((BinChar[0]   = 'B') or (BinChar[0]  = '*'  ))
                 and
                 ((szBBSFilter  = '*') or (szBBSFilter = ToBBS))
                 and
                 ((szNewsGroups = '' ) or (Pos(ToBBS,szNewsGroups) <> 0))
               ) then begin


                lpGroup := GetNews(lpNews,UpCaseStr(ToBBS));
                If lpGroup = Nil then begin
                   lpGroup         := IsrtNews(lpNews,UpCaseStr(ToBBS));
                   lpGroup^.szPost := 'y';
                   PutLog(3,'{AX25Mail} Created NEWS Instance for Group('+ToBBS+')');
                end; {endif}

                If (Trunc(MsgNro) < lpGroup^.iFirst) or (lpGroup^.iFirst = 0) then begin
                   lpGroup^.iFirst := Trunc(MsgNro);
                end; {endif}

                If (Trunc(MsgNro) > lpGroup^.iLast) or (lpGroup^.iLast = 0) then begin
                   lpGroup^.iLast := Trunc(MsgNro);
                end; {endif}

                inc(lpGroup^.iNum);

             end; {endif}

          end; {endif}
       end; {endif}

    end; {endwhile}

    PutLog(5,'{AX25Mail} Closing DMS File '+DMSFileName);
    Close(DMSFile);

end; { SearchDMS }
{*--->>  UpdateMBX <<<-------------------------------------------------*}
{* Update Mailbox                                                      *}
{*---------------------------------------------------------------------*}
Procedure UpdateMBX(szMailbox : String; szToken : String; iToken : Integer);
  Var
    UserINI     :       TINIFile;
begin { UpdateMBX }

     UserINI    := TINIFile.Create('HAMMAIL.INI');
     UserINI.WriteString(UpCaseStr(szMailBox),UpCaseStr(szToken),IntToStr(iToken));
     UserINI.Free;

end; { UpdateMBX }
{*--->>  ReleaseLL <<<-------------------------------------------------*}
{* This entry is used to release the memory LL                         *}
{*---------------------------------------------------------------------*}
Procedure ReleaseLL (Var lpHead : PtrMail);
  Var
    NPtr        :       PtrMail;
    PPtr        :       PtrMail;

begin { ReleaseLL }

  NPtr := lpHead;
  While NPtr <> Nil do begin
    PPtr := NPtr;
    NPtr := NPtr^.lpNext;
    Dispose(PPtr);
  end; {endwhile}
  lpHead := Nil;

  PutLog(3,'{AX25Mail} Mail Linked List Released');

end; { ReleaseLL }
(*--->>> Get           <<<----------------------------------------------*)
{* Get an instance of the linked list                                   *}
{*----------------------------------------------------------------------*}
Function Get (szMailBox : String) :   PtrMBox;

  Var
    NPtr        :       PtrMBox;

begin { LL }

    NPtr := lpMBx;
    While NPtr <> Nil do begin
       If NPtr.szMailBox = szMailBox then begin
          PutLog(5,'Get Success MBox Object '+szMailBox);
          Result := NPtr;
          Exit;
       end; {endif}
       NPtr := NPtr.lpNext;
    end; {endwhile}
    PutLog(1,'{AX25Mail} ERROR Get Failure MBox Object '+szMailBox);
    Result := Nil;

end; { LL }
(*--->>> Isrt          <<<----------------------------------------------*)
{*Isrt an instance of the linked list                                   *}
{*----------------------------------------------------------------------*}
Function Isrt (szMailBox : String) : PtrMBox ;
  Var
    NPtr        :       PtrMBox;
    PPtr        :       PtrMBox;

begin { LL }

  NPtr := lpMBx;
  PPtr := lpMBx;

  While PPtr <> Nil do begin
    If PPtr^.szMailBox = szMailBox then begin
       Result := PPtr;
       Exit;
    end; {endif}
    PPtr := NPtr^.lpNext;
    If PPtr <> Nil then begin
       NPtr := PPtr;
    end; {endif}
  end; {endwhile}

  New(PPtr);
  PPtr^.szMailBox   := szMailBox;
  PPtr^.szPass      := '';
  PPtr^.iMaxMail    := 1;
  PPtr^.iMaxNews    := 1;

  PPtr^.szGroup     := '';
  PPtr^.szArticle   := '';

  PPtr^.lpMailBox   := Nil;
  PPtr^.lpNews      := Nil;
  PPtr^.lpNext      := Nil;

  If lpMBx = Nil then begin
     lpMBx := PPtr;
     PutLog(5,'Isrt Success (First) MBox Object '+szMailBox);
  end else begin
     PutLog(5,'Isrt Success (Next) MBox Object '+szMailBox);
     NPtr^.lpNext := PPtr;
  end; {endif}

  Result := PPtr;

end; { LL }
(*--->>> Dlte          <<<----------------------------------------------*)
{*Dlte an instance of the linked list                                   *}
{*----------------------------------------------------------------------*}
Function Dlte (szMailBox : String) : PtrMBox;
  Var
    NPtr        :       PtrMBox;
    PPtr        :       PtrMBox;
begin { LL }

   If lpMBx = Nil then begin
      PutLog(1,'{AX25Mail} ERROR Dlte Failure (empty list) MBox Object '+szMailBox);
      Result := Nil;
      Exit;
   end; {endif}

   NPtr := lpMBx;
   PPtr := Nil;

   While NPtr <> Nil do begin
      If NPtr^.szMailBox = szMailBox then begin
         If PPtr <> Nil then begin
            PPtr^.lpNext := NPtr^.lpNext;
            PutLog(5,'Dlte Success (last) MBox Object '+szMailBox);
            Result := NPtr;
            Dispose(NPtr);
            Exit;
         end else begin
            If NPtr^.lpNext <> Nil then begin
               lpMBx := NPtr^.lpNext;
               PutLog(5,'Dlte Success (mid) MBox Object '+szMailBox);
               Result := NPtr;
               Dispose(NPtr);
               Exit;
            end else begin
               PutLog(5,'Dlte Success (last) MBox Object '+szMailBox);
               Result := NPtr;
               Dispose(NPtr);
               lpMBx := Nil;
               Exit;
            end; {endif}
         end; {endif}
         Result := Nil;
         Exit;
      end else begin
         PPtr := NPtr;
         NPtr := NPtr^.lpNext;
      end; {endif}
   end; {endwhile}

   PutLog(1,'{AX25Mail} ERROR Dlte failure (end of list) MBox Object '+szMailBox);
   Result := Nil;

end; { LL }
{*--->>  MakeRFC822 <<<------------------------------------------------*}
{* This creates a Header compliant with RFC-822                        *}
{*---------------------------------------------------------------------*}
Function MakeRFC822 (lpMail : PtrMail) : String;
  Var
    szHeader    :       String;
    szBuffer    :       String;
    MailFile    :       TextFile;
    szMail      :       String;
    szBBS       :       String;
    szAux       :       String;
    szToken     :       String;

begin { MakeRFC822 }

    szBuffer  := '';
    szBBS     := '';



    Assign(MailFile,lpMail^.szFile);
    Reset(MailFile);
    While not EOF(MailFile) do begin
       ReadLn(MailFile,szMail);
       If Copy(UpCaseStr(szMail),1,2) = 'R:' then begin
          szAux  := szMail;
          szBBS  := Parse(szAux);
          szBBS  := Parse(szAux);
          szToken:= szBBS;
          szToken:= PopBang(szBBS,'@');

          If Length(szBBS) > 3 then begin
             If szBBS[1] = ':' then begin
                szBBS := Copy(szBBS,2,Length(szBBS)-1);
             end;
          end else begin
             szBBS := '';
          end; {endif}
          PutLog(5,'{AX25Mail} Home BBS for Mail detected as '+szBBS);
       end;
       szBuffer := szBuffer + szMail + CRLF;
    end;
    Close(MailFile);

    if szBBS = '' then begin
       szBBS := szHomeBBS;
       PutLog(3,'{AX25Mail} Default Home BBS for Mail set as '+szBBS);
    end; {endif}

    szHeader    := '';
    szHeader    := szHeader + 'From: '+ lpMail^.szMailFrom + '@' + szBBS + CRLF;
    szHeader    := szHeader + 'To: '  + lpMail^.szMailTo   + '@' + lpMail^.szMailBBS + CRLF;
    szHeader    := szHeader + 'Message-Id: <' + lpMail^.szBid + '>' + CRLF;
    szHeader    := szHeader + 'Subject: '+ lpMail^.szSubject + CRLF;
    szHeader    := szHeader + CRLF;
    szHeader    := szHeader + szBuffer;

    Result      := szHeader;

end; { MakeRFC822 }
{*--->>  MakeNews   <<<------------------------------------------------*}
{* This creates a Header compliant with RFC-822                        *}
{*---------------------------------------------------------------------*}
Function MakeNews (lpMail : PtrMail) : String;
  Var
    szHeader    :       String;
    szBuffer    :       String;
    MailFile    :       TextFile;
    szMail      :       String;
    szBBS       :       String;
    szAux       :       String;
    szToken     :       String;

begin { MakeNews }

    szBuffer  := '';
    szBBS     := '';

    PutLog(3,'{AX25Mail} Spooling News <'+lpMail^.szBid+'> starting file('+lpMail^.szFile+')');

    Assign(MailFile,lpMail^.szFile);
{$I-}
    Reset(MailFile);
    If IOResult <> 0 then begin
       Result := '';
       Exit;
    end; {endif}
{$I+}
    While not EOF(MailFile) do begin
       ReadLn(MailFile,szMail);

       If Length(szMail) > 4 then begin
          If Copy(UpCaseStr(szMail),1,2) = 'R:' then begin
             szAux  := szMail;
             szBBS  := Parse(szAux);
             szBBS  := Parse(szAux);
             szToken:= szBBS;
             szToken:= PopBang(szBBS,'@');

             If Length(szBBS) > 3 then begin
                If szBBS[1] = ':' then begin
                   szBBS := Copy(szBBS,2,Length(szBBS)-1);
                end;
             end else begin
                szBBS := '';
             end; {endif}
             PutLog(5,'{AX25Mail} Home BBS for News detected as '+szBBS);
          end;
       end; {endif}
       szBuffer := szBuffer + szMail + CRLF;
    end;
    Close(MailFile);

    if szBBS = '' then begin
       szBBS := szHomeBBS;
       PutLog(3,'{AX25Mail} Default Home BBS for Mail set as '+szBBS);
    end; {endif}

    If lpMail^.szSubject = '' then begin
       lpMail^.szSubject := 'NO SUBJECT';
    end; {endif}

    PutLog(3,'{AX25Mail} Spooling News <'+lpMail^.szBid+'> build news buffer');
    szHeader    := '';
    szHeader    := szHeader + 'From: '+ lpMail^.szMailFrom + '@' + szBBS + CRLF;
    szHeader    := szHeader + 'NewsGroups: '  + lpMail^.szMailBBS + CRLF;
    szHeader    := szHeader + 'Message-Id: <' + lpMail^.szBid + '>' + CRLF;
    szHeader    := szHeader + 'Subject: '+ lpMail^.szSubject + CRLF;
    szHeader    := szHeader + CRLF;
    szHeader    := szHeader + szBuffer;
    PutLog(3,'{AX25Mail} Spooling News <'+lpMail^.szBid+'> completed');

    Result      := szHeader;

end; { MakeNews }
{*--->>  MailAuth  <<<-------------------------------------------------*}
{* This entry is used to authenticate a given mailbox                  *}
{*---------------------------------------------------------------------*}
Function MailAuth(sMailBox : PChar;sPass : PChar) : Boolean ; Far ; stdcall ;
  Var
   lpMailBox    :       PtrMBox;
   szMailBox    :       String;
   szAux        :       String;
   iSeq         :       Integer;
   MailBoxINI   :       TINIFile;
   szPassword   :       String;
   szPass       :       String;
   iMaxMail     :       Integer;
   szUser       :       String;

begin { MailAuth }

   If bEnabled = FALSE then begin
      PutLog(1,'{AX25Mail} ERROR Mail DLL is disabled');
      Result := FALSE;
      Exit;
   end; {endif}

   szMailBox  := PCharToStr(sMailBox);
   szMailBox  := GetStrZ(szMailBox);
   szMailBox  := UpCaseStr(szMailBox);

   If szMailBox = '' then begin
      PutLog(1,'{AX25Mail} ERROR Empty MailBox ('+szMailBox+')');
      Result := FALSE;
      Exit;
   end; {endif}


   PutLog(3,'{AX25Mail} Opening Mailbox '+szMailBox);

 {*----------------------------------------------*}
 {* Read the Configuration Data from HAMMAIL.INI *}
 {*----------------------------------------------*}

   szAux             := szMailBox;
   szUser            := szAux;
   MailBoxINI        := TINIFile.Create('HAMMAIL.INI');
   szPassword        := UpCaseStr(MailBoxINI.ReadString(szUser,'PASS',''));
   iMaxMail          := strtoint(MailBoxINI.ReadString(szUser,'MAIL','1'));
   MailBoxINI.Free;

   PutLog(3,'{AX25Mail} Mailbox '+szMailBox+' Password('+szPassword+') Last('+IntToStr(iMaxMail)+')');

   If iMaxMail < 1 then begin
      iMaxMail := 1;
   end else begin
      inc(iMaxMail);
   end; {endif}


   szPass     := PCharToStr(sPass);
   szPass     := GetStrZ(szPass);
   szPass     := UpCaseStr(szPass);

   If szPass = '' then begin
      PutLog(1,'{AX25Mail} ERROR Informed Password ('+szPass+') for Mailbox '+szMailBox+' Password('+szPassword+') Last('+IntToStr(iMaxMail)+')');
      Result := FALSE;
      Exit;
   end; {endif}


   If (szPassword = '') or (szPassword='*') or (szPassVfy='NOT') then begin
      PutLog(3,'{AX25Mail} WARNING No password verification performed');
   end else begin
      If szPassword = szPass then begin
         PutLog(3,'{AX25Mail} Informed Password ('+szPass+') for Mailbox '+szMailBox+' matches Password('+szPassword+') Last('+IntToStr(iMaxMail)+')');
      end else begin
         PutLog(1,'{AX25Mail} Informed Password ('+szPass+') for Mailbox '+szMailBox+' does not matches Password('+szPassword+') Last('+IntToStr(iMaxMail)+')');
         Result := FALSE;
         Exit;
      end; {endif}
   end; {endif}

   lpMailBox  := Isrt(szMailBox);

   lpMailBox^.szPass   := szPass;
   lpMailBox^.iMaxMail := iMaxMail;

   iSeq       := lpMailBox^.iMaxMail;

   ScanDMS(iSeq,'P',szMailBox,'*','',szMailDB,lpMailBox^.lpMailBox,szMailDir);

   Result     := TRUE;

end; { MailAuth }
{*--->>  MailStat  <<<-------------------------------------------------*}
{* This entry is used to return the number of mails pending            *}
{*---------------------------------------------------------------------*}
Function MailStat(sMailBox : PChar;Var iMail : DWORD;Var iSize : DWORD) : Boolean ; Far ; stdcall ;
  Var
    szMailBox   :       String;
    lpHead      :       PtrMail;
    NPtr        :       PtrMail;
    lpMail      :       PtrMBox;

begin { MailStat }

   If bEnabled = FALSE then begin
      Result := FALSE;
      Exit;
   end; {endif}

   szMailBox := PCharToStr(sMailBox);
   szMailBox := GetStrZ(szMailBox);
   szMailBox := UpCaseStr(szMailBox);

   lpMail    := Get(szMailBox);
   If lpMail = Nil then begin
      Result := FALSE;
      Exit;
   end; {endif}

   lpHead    := lpMail^.lpMailBox;

   iMail      := 0;
   iSize      := 0;

   NPtr       := lpHead;
   While NPtr <> Nil do begin
      inc(iMail);
      inc(iSize);
      NPtr := NPtr^.lpNext;
   end; {endwhile}

   Result     := TRUE;

end; { MailStat }
{*--->>  MailList  <<<-------------------------------------------------*}
{* This entry is used to return the list of mails found                *}
{*---------------------------------------------------------------------*}
Function MailList(sMailBox : PChar;Var lpData : PChar;Var iSize : DWORD) : Boolean ; Far ; stdcall ;

  Var
    szMailBox   :       String;
    lpHead      :       PtrMail;
    NPtr        :       PtrMail;
    lpMail      :       PtrMBox;
    szBuffer    :       String;
    iLength     :       Integer;
    szHeader    :       String;

begin { MailList }

   If bEnabled = FALSE then begin
      Result := FALSE;
      Exit;
   end; {endif}

   szMailBox := PCharToStr(sMailBox);
   szMailBox := GetStrZ(szMailBox);
   szMailBox := UpCaseStr(szMailBox);

   PutLog(3,'{AX25Mail} LIST Request for Mailbox '+szMailBox+' size('+IntToStr(iSize)+')');

   lpMail    := Get(szMailBox);
   If lpMail = Nil then begin
      Result := FALSE;
      Exit;
   end; {endif}

   lpHead    := lpMail^.lpMailBox;
   szBuffer  := '';

   NPtr       := lpHead;
   While NPtr <> Nil do begin

      szHeader := MakeRFC822(NPtr);
      iLength  := Length(szHeader);
      szBuffer := szBuffer + IntToStr(NPtr^.iMailNer)+' '+IntToStr(iLength)+CRLF;
      NPtr := NPtr^.lpNext;
   end; {endwhile}

   If DWORD(Length(szBuffer)) > iSize then begin
      Result := FALSE;
      Exit;
   end; {endif}

   PutLog(3,'{AX25Mail} Recovered LIST is ('+IntToStr(Length(szBuffer))+' bytes)'+CRLF+szBuffer);
   lpData := StrToPChar(szBuffer);
   iSize := Length(szBuffer);

   Result     := TRUE;

end; { MailList }
{*--->>  MailRetr  <<<-------------------------------------------------*}
{* This entry is used to retrieve a given mail                         *}
{*---------------------------------------------------------------------*}
Function MailRetr(sMailBox : PChar;iMail : DWORD;Var lpData : PChar;Var iSize : DWORD) : Boolean ; Far ; stdcall ;

  Var
    szMailBox   :       String;
    lpHead      :       PtrMail;
    NPtr        :       PtrMail;
    lpMail      :       PtrMBox;
    szBuffer    :       String;
    Index       :       Integer;
    szHeader    :       String;

begin { MailRetr }

   If bEnabled = FALSE then begin
      Result := FALSE;
      Exit;
   end; {endif}

   szMailBox := PCharToStr(sMailBox);
   szMailBox := GetStrZ(szMailBox);
   szMailBox := UpCaseStr(szMailBox);

   PutLog(3,'{AX25Mail} RETR Request for Mailbox '+szMailBox+' size('+IntToStr(iSize)+')');

   lpMail    := Get(szMailBox);
   If lpMail = Nil then begin
      Result := FALSE;
      Exit;
   end; {endif}

   lpHead    := lpMail^.lpMailBox;
   szBuffer  := '';

   NPtr       := lpHead;
   While NPtr <> Nil do begin

     If DWORD(NPtr^.iMailNer) = iMail then begin
        szHeader := MakeRFC822(NPtr);
        PutLog(5,'RFC822 Mail '+szHeader);
     end; {endif}
     NPtr := NPtr^.lpNext;
   end; {endwhile}

   If DWORD(Length(szHeader)) > iSize then begin
      Result := FALSE;
      Exit;
   end; {endif}

   PutLog(3,'{AX25Mail} Recovered RETR Mssg# '+IntToStr(iMail)+') is ('+IntToStr(Length(szHeader))+' bytes)'+CRLF+szBuffer);

   lpData := StrToPChar(szHeader);
   iSize := Length(szHeader);

   Result     := TRUE;

end; { MailRetr }
{*--->>  MailDele  <<<-------------------------------------------------*}
{* This entry is used to delete a given mail                           *}
{*---------------------------------------------------------------------*}
Function MailDele(sMailBox : PChar;iMail : DWORD) : Boolean ; Far ; stdcall ;

  Var
    szMailBox   :       String;
    lpHead      :       PtrMail;
    NPtr        :       PtrMail;
    lpMail      :       PtrMBox;
    szBuffer    :       String;

begin { MailDele }

   If bEnabled = FALSE then begin
      Result := FALSE;
      Exit;
   end; {endif}

   szMailBox := PCharToStr(sMailBox);
   szMailBox := GetStrZ(szMailBox);
   szMailBox := UpCaseStr(szMailBox);

   PutLog(3,'{AX25Mail} MailBox('+szMailBox+') DELE Mssg# '+IntToStr(iMail)+')'+CRLF);

   lpMail    := Get(szMailBox);
   If lpMail = Nil then begin
      Result := FALSE;
      Exit;
   end; {endif}

   lpHead    := lpMail^.lpMailBox;
   szBuffer  := '';

   NPtr       := lpHead;
   While NPtr <> Nil do begin

     If DWORD(NPtr^.iMailNer) = iMail then begin
        PutLog(3,'{AX25Mail} Message detected, delete OK returned');
        UpdateMBX(szMailBox,'MAIL',NPtr^.iMailSeq);
        Result := TRUE;
        Exit;
     end; {endif}
     NPtr := NPtr^.lpNext;
   end; {endwhile}

   PutLog(1,'{AX25Mail} ERROR Message NOT detected, delete failure returned');
   Result     := FALSE;

end; { MailDele }
{*--->>  MailClose <<<-------------------------------------------------*}
{* This entry is used to close a given MailBox                         *}
{*---------------------------------------------------------------------*}
Function MailClose(sMailBox : PChar) : Boolean ; Far ; stdcall ;

  Var
    szMailBox   :       String;
    lpMail      :       PtrMBox;
begin { MailClose }

   If bEnabled = FALSE then begin
      Result := FALSE;
      Exit;
   end; {endif}

   szMailBox := PCharToStr(sMailBox);
   szMailBox := GetStrZ(szMailBox);
   szMailBox := UpCaseStr(szMailBox);

   lpMail    := Get(szMailBox);
   If lpMail = Nil then begin
      PutLog(1,'{AX25Mail} ERROR Unable to close mailbox '+szMailbox);
      Result := TRUE;
      Exit;
   end; {endif}

   If lpMail^.lpMailBox <> Nil then begin
      ReleaseLL(lpMail^.lpMailBox);
   end; {endif}

   If lpMail^.lpNews <> Nil then begin
      ReleaseNewsLL(lpMail^.lpNews);
   end; {endif}

   Dlte(szMailBox);
   PutLog(3,'{AX25Mail} Mailbox '+szMailbox+' processing complete, released');

   Result := TRUE;

end; { MailClose }
{*--->>  NewsOpen  <<<-------------------------------------------------*}
{* This entry is used to authenticate a given mailbox                  *}
{*---------------------------------------------------------------------*}
Function NewsOpen(sMailBox : PChar; sPass : PChar) : Boolean ; Far ; stdcall ;
  Var
   lpMbx        :       PtrMBox;
   szMailBox    :       String;
   MailBoxINI   :       TINIFile;
   szPassword   :       String;
   szPass       :       String;
   szAux        :       String;
   szNewsGroups :       String;
   szUser       :       String;

begin { NewsOpen }

   If bEnabled = FALSE then begin
      PutLog(1,'{AX25Mail} ERROR Mail DLL is disabled');
      Result := FALSE;
      Exit;
   end; {endif}

   szMailBox  := PCharToStr(sMailBox);
   szMailBox  := GetStrZ(szMailBox);
   szMailBox  := UpCaseStr(szMailBox);

   szPass     := PCharToStr(sPass);
   szPass     := GetStrZ(szPass);

   If (szMailBox = '') or (szPass = '') then begin
      PutLog(1,'{AX25Mail} ERROR Wrong User Id ('+szMailBox+')');
      Result := FALSE;
      Exit;
   end; {endif}

 {*----------------------------------------------*}
 {* Read the Configuration Data from HAMMAIL.INI *}
 {*----------------------------------------------*}

   szAux             := szMailBox;
   szUser            := PopBang(szAux,'_');

   MailBoxINI        := TINIFile.Create('HAMMAIL.INI');
   szPassword        := MailBoxINI.ReadString(szUser,'PASS','');
   szNewsGroups      := UpCaseStr(MailBoxINI.ReadString(szUser,'NEWSGROUPS',''));
   MailBoxINI.Free;

   PutLog(3,'{AX25Mail} Mailbox '+szMailBox+' Password('+szPassword+')');

   If szPassword = '' then begin
      PutLog(1,'{AX25Mail} ERROR Not informed Password for News ID '+szMailBox);
      Result := FALSE;
      Exit;
   end; {endif}

   If szPass <> szPassword then begin
      PutLog(1,'{AX25Mail} ERROR Security violation ID '+szMailBox);
      Result := FALSE;
      Exit;
   end; {endif}

   lpMbx := Isrt(szMailBox+'@NEWS');

   lpMbx^.szPass       := szPass;
   lpMbx^.iMaxMail     := 0;
   lpMbx^.iMaxNews     := 0;
   lpMbx^.szGroup      := '';
   lpMbx^.szArticle    := szNewsGroups;
   lpMbx^.lpMailBox    := Nil;
   lpMbx^.lpNews       := Nil;

   Result     := TRUE;
end; { NewsOpen }
{*--->>  NewsLst   <<<-------------------------------------------------*}
{* This entry is used to authenticate a given mailbox                  *}
{*---------------------------------------------------------------------*}
Function NewsLst(sMailBox : PChar;Var lpData : PChar ; Var iSize : DWORD) : Boolean ; Far ; stdcall ;

  Var
   lpMbx        :       PtrMBox;
   szMailBox    :       String;
   iSeq         :       Integer;
   szBuffer     :       String;
   NPtr         :       PtrNews;
   szAux        :       String;

begin { NewsLst }

   If bEnabled = FALSE then begin
      PutLog(1,'{AX25Mail} ERROR Mail DLL is disabled');
      Result := FALSE;
      Exit;
   end; {endif}

   iSeq   := 1;
   szMailBox := PCharToStr(sMailBox);
   szMailBox := GetStrZ(szMailBox);
   szMailBox := UpCaseStr(szMailBox);

   lpMbx     := Get(szMailBox+'@NEWS');
   If lpMbx = Nil then begin
      PutLog(1,'{AX25Mail} ERROR Mailbox '+szMailBox+'@NEWS not found');
      Result := FALSE;
      Exit;
   end; {endif}

   If lpMbx^.lpNews <> Nil then begin
      FreeNews(lpMbx^.lpNews);
   end; {endif}

   SearchDMS(iSeq,'B','*',lpMbx^.szArticle,szMailDB,lpMbx^.lpNews,szMailDir);

   NPtr     := lpMbx^.lpNews;
   szBuffer := '';

   While NPtr <> Nil  do begin
      szBuffer := szBuffer + NPtr^.szNewsId+' '+IntToStr(NPtr^.iLast)+' '+IntToStr(NPtr^.iFirst)+' '+NPtr^.szPost+CRLF;
      NPtr     := NPtr^.lpNext;
   end; {endwhile}

   If DWORD(Length(szBuffer)) > iSize then begin
      PutLog(1,'{AX25Mail} ERROR Memory allocation error while processing NEWS LIST command');
      Result := FALSE;
      Exit;
   end; {endif}

   lpData:= StrToPChar(szBuffer);
   iSize := Length(szAux);

   Result                   := TRUE;

end; { NewsLst }
{*--->>  NewsGroup <<<-------------------------------------------------*}
{* This entry is used to authenticate a given mailbox                  *}
{*---------------------------------------------------------------------*}
Function NewsGroup(sMailBox : PChar;Var lpGroup: PChar;Var iFirst : DWORD;Var iLast : DWORD;Var iNum : DWORD;Var lpArticle : PChar) : Boolean ; Far; stdcall;

  Var
   lpMbx        :       PtrMBox;
   szMailBox    :       String;
   iSeq         :       Integer;
   NPtr         :       PtrNews;
   szGroup      :       String;
   MPtr         :       PtrMail;
   PPtr         :       PtrMail;

begin { NewsGroup }

   If bEnabled = FALSE then begin
      PutLog(1,'{AX25Mail} ERROR Mail DLL is disabled');
      Result := FALSE;
      Exit;
   end; {endif}

   iSeq   := 1;

   szMailBox := PCharToStr(sMailBox);
   szMailBox := GetStrZ(szMailBox);
   szMailBox := UpCaseStr(szMailBox);

   szGroup   := PCharToStr(lpGroup);
   szGroup   := GetStrZ(szGroup);
   szGroup   := UpCaseStr(szGroup);

   lpMbx     := Get(szMailBox+'@NEWS');
   If lpMbx = Nil then begin
      PutLog(1,'{AX25Mail} ERROR Memory allocation error while doing GROUP '+szGroup);
      Result := FALSE;
      Exit;
   end; {endif}

   If Pos(UpCaseStr(szGroup),UpCaseStr(lpMbx^.szArticle)) = 0 then begin
      PutLog(1,'{AX25Mail} ERROR Not authorized for GROUP '+szGroup);
      Result := FALSE;
      Exit;
   end; {endif}

   If lpMbx^.lpNews = Nil then begin
      iSeq := 1;
      SearchDMS(iSeq,'B',szGroup,lpMbx^.szArticle,szMailDB,lpMbx^.lpNews,szMailDir);
   end; {endif}

   If lpMbx^.lpMailBox <> Nil then begin
      MPtr := lpMbx^.lpMailbox;
      While MPtr <> Nil do begin
         PPtr := MPtr;
         MPtr := MPtr^.lpNext;
         Dispose(PPtr);
      end; {endwhile}
      lpMbx^.lpMailBox := Nil;
   end; {endif}

   lpMbx^.szGroup := szGroup;

   NPtr := lpMbx^.lpNews;
   While NPtr <> Nil do begin
      If NPtr^.szNewsId = szGroup then begin
         iFirst := NPtr^.iFirst;
         iLast  := NPtr^.iLast;
         iNum   := NPtr^.iNum;
         Result := TRUE;
         Exit;
      end; {endif}
      NPtr := NPtr^.lpNext;
   end; {endwhile}

   iFirst:= 0;
   iLast := 0;
   iNum  := 0;

   Result:= FALSE;

end; { NewsGroup }
{*--->>  NewsXOver <<<-------------------------------------------------*}
{* This entry is used to authenticate a given mailbox                  *}
{*---------------------------------------------------------------------*}
Function NewsXOver(sMailBox : PChar;Var lpData : PChar;Var iSize : DWORD;iFirst : DWORD;iLast : DWORD) : Boolean ; Far; stdcall;

  Var
   lpMbx        :       PtrMBox;
   szMailBox    :       String;
   iSeq         :       Integer;
   szBuffer     :       String;
   NPtr         :       PtrMail;
   szGroup      :       String;

begin { NewsXOver }

   If bEnabled = FALSE then begin
      PutLog(1,'{AX25Mail} ERROR Mail DLL is disabled');
      Result := FALSE;
      Exit;
   end; {endif}

   If sMailBox = '' then begin
      PutLog(1,'{AX25Mail} ERROR Mailbox not indicated');
      Result := FALSE;
      Exit;
   end; {endif}

   iSeq   := 1;
   szMailBox := PCharToStr(sMailBox);
   szMailBox := GetStrZ(szMailBox);
   szMailBox := UpCaseStr(szMailBox);

   lpMbx     := Get(szMailBox+'@NEWS');
   If lpMbx = Nil then begin
      PutLog(1,'{AX25Mail} ERROR Memory allocation error while doing GROUP '+szGroup);
      Result := FALSE;
      Exit;
   end; {endif}

   If lpMbx^.szGroup = '' then begin
      PutLog(1,'{AX25Mail} ERROR No Group currently selected');
      Result := FALSE;
      Exit;
   end; {endif}

   If lpMbx^.lpMailBox = Nil then begin
      iSeq := 1;
      ScanDMS(iSeq,'B','*',lpMbx^.szGroup,'',szMailDB,lpMbx^.lpMailBox,szMailDir);
   end; {endif}

   szBuffer := '';
   NPtr     := lpMbx^.lpMailBox;
   While NPtr <> Nil do begin
      If NPtr^.szMailBBS = lpMbx^.szGroup then begin
         If ((DWORD(NPtr^.iMailSeq) >= iFirst) and (DWORD(NPtr^.iMailSeq) <= iLast)) then begin
            szBuffer := szBuffer + IntToStr(NPtr^.iMailSeq) + TAB +
                                   NPtr^.szSubject          + TAB +
                                   NPtr^.szMailFrom         + TAB + TAB +
                             '<' + NPtr^.szBid + '> '       + TAB + TAB + TAB + TAB + CRLF;
         end; {endif}
      end; {endif}
      NPtr := NPtr^.lpNext;
   end; {endwhile}

   If DWORD(Length(szBuffer)) > iSize then begin
      PutLog(1,'{AX25Mail} ERROR Memory allocation error while processing NEWS LIST command');
      Result := FALSE;
      Exit;
   end; {endif}

   lpData:= StrToPChar(szBuffer);
   iSize := Length(szBuffer);
   Result:= TRUE;

end; { NewsXOver }
{*--->>  NewsRead  <<<-------------------------------------------------*}
{* This entry is used to authenticate a given mailbox                  *}
{*---------------------------------------------------------------------*}
Function NewsRead(sMailBox : PChar;lpArticle : PChar;Var lpData : PChar; Var iSize : DWORD;Var lpBid : PChar) : Boolean ; Far; stdcall;

  Var
   lpMbx        :       PtrMBox;
   szMailBox    :       String;
   iSeq         :       Integer;
   szBuffer     :       String;
   NPtr         :       PtrMail;
   szGroup      :       String;
   szArticle    :       String;

begin { NewsRead }

   If bEnabled = FALSE then begin
      PutLog(1,'{AX25Mail} ERROR Mail DLL is disabled');
      Result := FALSE;
      Exit;
   end; {endif}

   PutLog(5,'{AX25Mail} Processing ARTICLE');

   szMailBox := PCharToStr(sMailBox);
   szMailBox := GetStrZ(szMailBox);
   szMailBox := UpCaseStr(szMailBox);

   lpMbx     := Get(szMailBox+'@NEWS');
   If lpMbx = Nil then begin
      PutLog(1,'{AX25Mail} ERROR Memory allocation error while doing GROUP '+szGroup);
      Result := FALSE;
      Exit;
   end; {endif}

   If lpMbx^.szGroup = '' then begin
      PutLog(1,'{AX25Mail} ERROR No Group currently selected');
      Result := FALSE;
      Exit;
   end; {endif}

   szArticle := PCharToStr(lpArticle);
   szArticle := GetStrZ(szArticle);
   iSeq      := 1;

   PutLog(3,'{AX25Mail} ARTICLE '+szArticle+'('+IntToStr(iSeq)+') '+lpMbx^.szGroup+' for Id('+szMailBox+')');
   If lpMbx^.lpMailBox = Nil then begin
      ScanDMS(iSeq,'B','*',lpMbx^.szGroup,'',szMailDB,lpMbx^.lpMailBox,szMailDir);
   end; {endif}

   szBuffer := '';
   NPtr     := lpMbx^.lpMailBox;
   While NPtr <> Nil do begin

      If NPtr^.szMailBBS = lpMbx^.szGroup then begin
         If NPtr^.iMailSeq = StrToInt(szArticle) then begin
            szBuffer := '';
            PutLog(3,'{AX25Mail} ARTICLE Recovered now spooling ');
            szBuffer := MakeNews(NPtr);

            If DWORD(Length(szBuffer)) > iSize then begin
               PutLog(1,'{AX25Mail} ERROR Memory allocation error while processing NEWS LIST command');
               Result := FALSE;
               Exit;
            end; {endif}

            lpData:= StrToPChar(szBuffer);
            iSize := Length(szBuffer);
            lpBid := StrToPChar(NPtr^.szBid);
            Result:= TRUE;
            Exit;
         end; {endif}
      end; {endif}
      NPtr := NPtr^.lpNext;
   end; {endwhile}

   PutLog(3,'{AX25Mail} ERROR Group '+lpMbx^.szGroup+' Article('+szArticle+') for Id('+szMailBox+') Not Found');
   Result := FALSE;

end; { NewsXOver }
{*--->>  Export Table <<-----------------------------------------------*}
{* Export Table                                                        *}
{* This section exports the functions of the DLL                       *}
{*                                                                     *}
{*                                                                     *}
{*                                                                     *}
{*---------------------------------------------------------------------*}
exports

      MailAuth         index  1,
      MailStat         index  2,
      MailList         index  3,
      MailRetr         index  4,
      MailDele         index  5,
      MailClose        index  6,
      NewsLst          index  7,
      NewsGroup        index  8,
      NewsXOver        index  9,
      NewsRead         index 10,
      NewsOpen         index 11;

{*--->>  Initialization <<---------------------------------------------*}
{* Initialization Code                                                 *}
{*                                                                     *}
{*                                                                     *}
{*                                                                     *}
{*---------------------------------------------------------------------*}
begin

   bEnabled  := TRUE;
   lpMbx     := Nil;

 {*----------------------------------------------*}
 {* Read the Configuration Data from HAMMAIL.INI *}
 {*----------------------------------------------*}


   HAMMAILINI        := TINIFile.Create('HAMMAIL.INI');

   DEBUGMODE         := UpCaseStr(HAMMAILINI.ReadString('MAIN','DEBUG','YES'));
   szLogFile         := UpCaseStr(HAMMAILINI.ReadString('MAIN','LOGFILE','HAMMAIL.LOG'));
   szHomeBBS         := UpCaseStr(HAMMAILINI.ReadString('MAIN','HOMEBBS',''));
   TRACELEVEL        := strtoint(HAMMAILINI.ReadString('MAIN','TRACE','10'));
   szMailDB          := UpCaseStr(HAMMAILINI.ReadString('AX25MAIL','MAILBOX',''));
   szMailDir         := UpCaseStr(HAMMAILINI.ReadString('AX25MAIL','MAILDIR',''));
   szPassVfy         := UpCaseStr(HAMMAILINI.ReadString('AX25MAIL','PASSVFY','NOT'));

   HAMMAILINI.Free;

   If (szMailDB  = '') or (szMailDir = '') then begin
      PutLog(1,'{AX25Mail} ERROR Mail DB or Mail Directory not indicated, DLL is disabled');
      bEnabled := FALSE;
   end else begin
      bEnabled := TRUE;
   end; {endif}

   If szHomeBBS = '' then begin
      PutLog(1,'{AX25Mail} ERROR Home BBS has not been defined, DLL is disabled');
      bEnabled := FALSE;
   end; {endif}

   If (szLogFile = '') and (DEBUGMODE = 'YES') then begin
      PutLog(1,'{AX25Mail} WARNING DEBUG is forced OFF because no Log File has been informed');
      DEBUGMODE := 'NOT';
   end; {endif}

   If szMailDir[Length(szMailDir)] <> '\' then begin
      szMailDir := szMailDir + '\';
   end; {endif}

   PutLog(3,'--->>> Start of AX25MAIL.DLL Parameters <<<--- ');
   PutLog(3,'DEBUG Mode     '+DEBUGMODE);
   PutLog(3,'TRACE Level    '+inttostr(TRACELEVEL));
   PutLog(3,'Log File       '+szLogFile);
   PutLog(3,'Mail DB        '+szMailDB);
   PutLog(3,'Mail Dir       '+szMailDir);
   PutLog(3,'Pass Verify    '+szPassVfy);
   PutLog(3,'--->>> End of Init Parameters <<<--- ');


   PutLog(1,'{AX25Mail} Mail DLL initialization completed');
end.
